﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GoniometricFunctions
{
    abstract class AbstractFunction : IFunction
    {
        public double A { get; set; }
        public double B { get; set; }
        public double C { get; set; }

        protected AbstractFunction(double a, double b, double c)
        {
            A = a;
            B = b;
            C = c;
        }

        public static IFunction GetFunction(string func, double a, double b, double c)
        {
            switch (func)
            {
                case "sin": return new Sine(a, b, c);
                case "cos": return new Cosine(a, b, c);
                case "tan":
                case "tg": return new Tangent(a, b, c);
                case "cotg":
                case "cotan": return new Cotangent(a, b, c);
                default: return new DummyFunc();
            }
        }

        public abstract double GetFuncValue(double x);
    }

    class Sine : AbstractFunction
    {
        public Sine(double a, double b, double c)
            : base(a, b, c)
        {
        }

        public override double GetFuncValue(double x)
        {
            return (double)(A * Math.Sin(Math.PI * B * x) + C);
        }
    }

    class Cosine : AbstractFunction
    {
        public Cosine(double a, double b, double c)
            : base(a, b, c)
        {
        }

        public override double GetFuncValue(double x)
        {
            return (double)(A * Math.Cos(Math.PI * B * x) + C);
        }
    }

    class Tangent : AbstractFunction
    {
        public Tangent(double a, double b, double c)
            : base(a, b, c)
        {
        }

        public override double GetFuncValue(double x)
        {
            return (double)(A * Math.Tan(Math.PI * B * x) + C);
        }
    }

    class Cotangent : AbstractFunction
    {
        public Cotangent(double a, double b, double c)
            : base(a, b, c)
        {
        }

        public override double GetFuncValue(double x)
        {
            double cotg = (Math.Tan(Math.PI * B * x) == 0) ? double.NaN : (double)(1 / Math.Tan(Math.PI * B * x));
            return (double)(A * cotg + C);
        }
    }

    class DummyFunc : AbstractFunction
    {
        public DummyFunc() : base(0,0,0)
        {}
        
        public override double GetFuncValue(double x)
        {
            return 0;
        }
    }

}
